@@ -1,7 +1,7 @@
 /*
  * rlm_eap.c  contains handles that are called from modules.
  *
- * Version:     $Id: rlm_eap.c,v 1.53 2007/12/16 08:31:30 aland Exp $
+ * Version:     $Id: rlm_eap.c,v 1.55 2008/02/10 09:18:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: rlm_eap.c,v 1.53 2007/12/16 08:31:30 aland Exp $")
+RCSID("$Id: rlm_eap.c,v 1.55 2008/02/10 09:18:10 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -79,10 +79,8 @@ static int eap_handler_cmp(const void *a, const void *b)
 	const EAP_HANDLER *one = a;
 	const EAP_HANDLER *two = b;
 
-#if 0
 	if (one->eap_id < two->eap_id) return -1;
 	if (one->eap_id > two->eap_id) return +1;
-#endif
 
 	rcode = fr_ipaddr_cmp(&one->src_ipaddr, &two->src_ipaddr);
 	if (rcode != 0) return rcode;
@@ -118,6 +116,7 @@ static int eap_instantiate(CONF_SECTION *cs, void **instance)
 		inst->rand_pool.randrsl[i] = fr_rand();
 	}
 	fr_randinit(&inst->rand_pool, 1);
+	inst->rand_pool.randcnt = 0;
 
 	/* Load all the configured EAP-Types */
 	num_types = 0;
@@ -355,9 +354,8 @@ static int eap_authenticate(void *instance, REQUEST *request)
 	 *	Add to the list only if it is EAP-Request, OR if
 	 *	it's LEAP, and a response.
 	 */
-	if ((handler->eap_ds->request->code == PW_EAP_REQUEST) &&
-	    (handler->eap_ds->request->type.type >= PW_EAP_MD5)) {
-		eaplist_add(inst, handler);
+	if (((handler->eap_ds->request->code == PW_EAP_REQUEST) &&
+	    (handler->eap_ds->request->type.type >= PW_EAP_MD5)) ||
 
 		/*
 		 *	LEAP is a little different.  At Stage 4,
@@ -368,12 +366,16 @@ static int eap_authenticate(void *instance, REQUEST *request)
 		 *	At stage 6, LEAP sends an EAP-Response, which
 		 *	isn't put into the list.
 		 */
-	} else if ((handler->eap_ds->response->code == PW_EAP_RESPONSE) &&
-		   (handler->eap_ds->response->type.type == PW_EAP_LEAP) &&
-		   (handler->eap_ds->request->code == PW_EAP_SUCCESS) &&
-		   (handler->eap_ds->request->type.type == 0)) {
+	    ((handler->eap_ds->response->code == PW_EAP_RESPONSE) &&
+	     (handler->eap_ds->response->type.type == PW_EAP_LEAP) &&
+	     (handler->eap_ds->request->code == PW_EAP_SUCCESS) &&
+	     (handler->eap_ds->request->type.type == 0))) {
 
-		eaplist_add(inst, handler);
+		if (!eaplist_add(inst, handler)) {
+			eap_fail(handler);
+			eap_handler_free(handler);
+			return RLM_MODULE_FAIL;
+		}
 
 	} else {
 		DEBUG2("  rlm_eap: Freeing handler");
